use std::{path::PathBuf, thread};

use anyhow::Result;
use app::App;
use clap::Parser;
use emulator::EmulatorBuilder;
use winit::event_loop::{ControlFlow, EventLoop};

mod app;
mod audio;
mod emulator;
mod renderer;
mod shrooms_vb_core;

#[derive(Parser)]
struct Args {
    rom: PathBuf,
}

fn main() -> Result<()> {
    let args = Args::parse();

    let (builder, client) = EmulatorBuilder::new();
    let builder = builder.with_rom(&args.rom);
    thread::spawn(move || {
        let mut emulator = builder.build().unwrap();
        emulator.run();
    });

    let event_loop = EventLoop::new().unwrap();
    event_loop.set_control_flow(ControlFlow::Poll);
    event_loop.run_app(&mut App::new(client))?;
    Ok(())
}
